;;; ----------------------
;;; Normalized polynomials
;;; ----------------------

(in-package "NPOL")

(include-book "upol")
(include-book "ordinal-ordering")

;;; ---------
;;; Functions
;;; ---------

;;; Recognizer.

(defun polynomialp (p)
  (declare (xargs :guard t))
  (and (UPOL::polynomialp p) (UPOL::nfp p)))

;;; ...

;;; Polynomial ordering.

(defun < (p q)
  (declare (xargs :guard (and (polynomialp p) (polynomialp q))))
  (cond ((or (endp p) (endp q))
         (not (endp q)))
	((MON::=T (first p) (first q))
         (< (rest p) (rest q)))
        (t
         (MON::< (first p) (first q)))))

;;; Ordinal embedding.

(defun polynomial->e0-ordinal (p)
  (declare (xargs :guard (polynomialp p)))
  (if (endp p)
      0
    (cons (monomial->e0-ordinal (first p))
          (polynomial->e0-ordinal (rest p)))))

;;; ----------
;;; Properties
;;; ----------

;;; ...

;;; The polynomial ordering is a partial strict order.

(defthm |~(p < p)|
  (not (< p p)))

(defthm |p < q & q < r => p < r|
  (implies (and (< p q) (< q r))
	   (< p r)))

;;; Antisymmetry of the polynomial ordering.

(defthm |p < q => ~(q < p)|
  (implies (< p q)
	   (not (< q p)))
  :hints (("Goal"
	   :in-theory (disable |p < q & q < r => p < r|)
	   :use (:instance |p < q & q < r => p < r| (r p)))))

;;; Correctness of the ordinal embedding.

(defthm correctness-of-polynomial->e0-ordinal
  (implies (polynomialp p)
           (e0-ordinalp (polynomial->e0-ordinal p))))

;;; Well-foundedness of the polynomial ordering.

(defthm well-foundedness-of-<
  (and (implies (polynomialp p)
                (e0-ordinalp (polynomial->e0-ordinal p)))
       (implies (and (polynomialp p) (polynomialp q)
                     (< p q))
                (e0-ord-< (polynomial->e0-ordinal p)
                          (polynomial->e0-ordinal q))))
  :rule-classes :well-founded-relation)
