;;; -----
;;; Terms
;;; -----

(in-package "TER")

(encapsulate
  ((termp (a) t)
   ;;; ...
   (< (a b) t)
   (term->e0-ordinal (a) t))

  ;;; ---------------
  ;;; Local witnesses
  ;;; ---------------

  ;;; Recognizer.

  (local
    (defun termp (a)
      (acl2-numberp a)))

  ;;; ...

  ;;; Term ordering.

  (local
    (defun < (a b)
      (e0-ord-< (nfix a) (nfix b))))

  ;;; Ordinal embedding.
  
  (local
    (defun term->e0-ordinal (a)
      (+ (nfix a) 1)))

  ;;; ------
  ;;; Axioms
  ;;; ------

  ;;; ...

  ;;; The term ordering is a partial strict order.

  (defthm |~(a < a)|
    (not (< a a)))

  (defthm |a < b & b < c => a < c|
    (implies (and (< a b) (< b c)) (< a c)))

  ;;; The ordinal embedding does not produce 0.

  (defthm |~(term->e0-ordinal(a) = 0)|
    (not (equal (term->e0-ordinal a) 0)))

  ;;; Well-foundedness of the term ordering.

  (defthm well-foundedness-of-<
    (and (implies (termp a)
		  (e0-ordinalp (term->e0-ordinal a)))
	 (implies (and (termp a) (termp b)
		       (< a b))
		  (e0-ord-< (term->e0-ordinal a) (term->e0-ordinal b))))
    :rule-classes (:rewrite :well-founded-relation)))
